@@ -16,29 +16,30 @@ from lems.parser.LEMS import LEMSFileParser
 from lems.base.errors import ModelError
 from lems.base.errors import SimBuildError
 
-from lems.model.fundamental import Dimension,Unit
+from lems.model.fundamental import Dimension,Unit,Include
 from lems.model.component import Constant,ComponentType,Component,FatComponent
-from lems.model.simulation import Run,Record,DataDisplay,DataWriter
+from lems.model.simulation import Run,Record,EventRecord,DataDisplay,DataWriter,EventWriter
 from lems.model.structure import With,EventConnection,ChildInstance,MultiInstantiate
 
 import xml.dom.minidom as minidom
 
 import logging
 
-logging.basicConfig(level=logging.DEBUG)
-
 class Model(LEMSBase):
     """
     Stores a model.
     """
+
+    logging.basicConfig(level=logging.INFO)
+
     target_lems_version = '0.7.3'
     branch = 'development'
-    schema_location = 'https://raw.github.com/LEMS/LEMS/{0}/Schemas/LEMS/LEMS_v{1}.xsd'.format(branch, target_lems_version)
+    schema_location = 'https://raw.githubusercontent.com/LEMS/LEMS/{0}/Schemas/LEMS/LEMS_v{1}.xsd'.format(branch, target_lems_version)
     #schema_location = '/home/padraig/LEMS/Schemas/LEMS/LEMS_v%s.xsd'%target_lems_version
     
     debug = False
     
-    def __init__(self, include_includes='True'):
+    def __init__(self, include_includes=True, fail_on_missing_includes=True):
         """
         Constructor.
         """
@@ -47,6 +48,10 @@ class Model(LEMSBase):
         """ List of targets to be run on startup.
         @type: list(str) """
         
+        self.includes = Map()
+        """ Dictionary of includes defined in the model.
+        @type: dict(str -> lems.model.fundamental.Include """
+        
         self.dimensions = Map()
         """ Dictionary of dimensions defined in the model.
         @type: dict(str -> lems.model.fundamental.Dimension """
@@ -86,6 +91,10 @@ class Model(LEMSBase):
         self.include_includes = include_includes
         """ Whether to include LEMS definitions in <Include> elements
         @type: boolean """
+        
+        self.fail_on_missing_includes = fail_on_missing_includes
+        """ Whether to raise an Exception when a file in an <Include> element is not found
+        @type: boolean """
 
     def add_target(self, target):
         """
@@ -98,6 +107,16 @@ class Model(LEMSBase):
         
         self.targets.append(target)
         
+    def add_include(self, include):
+        """
+        Adds an include to the model.
+
+        @param include: Include to be added.
+        @type include: lems.model.fundamental.Include
+        """
+
+        self.includes[include.file] = include
+        
     def add_dimension(self, dimension):
         """
         Adds a dimension to the model.
@@ -125,8 +144,14 @@ class Model(LEMSBase):
         @param component_type: Component type to be added.
         @type component_type: lems.model.fundamental.ComponentType
         """
-
-        self.component_types[component_type.name] = component_type
+        name = component_type.name
+        
+        # To handle colons in names in LEMS
+        if ':' in name:
+            name = name.replace(':', '_')
+            component_type.name = name
+            
+        self.component_types[name] = component_type
 
     def add_component(self, component):
         """
@@ -165,7 +190,9 @@ class Model(LEMSBase):
         @param child: Child object to be added.
         """
 
-        if isinstance(child, Dimension):
+        if isinstance(child, Include):
+            self.add_include(child)
+        elif isinstance(child, Dimension):
             self.add_dimension(child)
         elif isinstance(child, Unit):
             self.add_unit(child)
@@ -224,7 +251,11 @@ class Model(LEMSBase):
                         else:
                             if self.debug: print("Already included: %s"%path)
                             return
-            raise Exception('Unable to open ' + path)
+            msg = 'Unable to open ' + path
+            if self.fail_on_missing_includes:
+                raise Exception(msg)
+            elif self.debug: 
+                print(msg)
             
     def import_from_file(self, filepath):
         """
@@ -241,12 +272,9 @@ class Model(LEMSBase):
         with open(filepath) as f:
             parser.parse(f.read())
         
-    def export_to_file(self, filepath, level_prefix = '  '):
+    def export_to_dom(self):
         """
-        Exports this model to a file.
-
-        @param filepath: File to be exported to.
-        @type filepath: str
+        Exports this model to a DOM.
         """
         namespaces = 'xmlns="http://www.neuroml.org/lems/%s" ' + \
                      'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ' + \
@@ -256,6 +284,9 @@ class Model(LEMSBase):
 
         xmlstr = '<Lems %s>'%namespaces
 
+        for include in self.includes:
+            xmlstr += include.toxml()
+
         for target in self.targets:
             xmlstr += '<Target component="{0}"/>'.format(target)
 
@@ -276,10 +307,23 @@ class Model(LEMSBase):
             
         xmlstr += '</Lems>'
 
-        xmlstr = minidom.parseString(xmlstr).toprettyxml(level_prefix, '\n',)
+        xmldom = minidom.parseString(xmlstr)
+        return xmldom
 
-        with open(filepath, 'w') as f:
-            f.write(xmlstr)
+    def export_to_file(self, filepath, level_prefix = '  '):
+        """
+        Exports this model to a file.
+
+        @param filepath: File to be exported to.
+        @type filepath: str
+        """
+        xmldom = self.export_to_dom()
+        xmlstr = xmldom.toprettyxml(level_prefix, '\n',)
+
+
+        f = open(filepath, 'w')
+        f.write(xmlstr)
+        f.close()
 
     def resolve(self):
         """
@@ -346,10 +390,15 @@ class Model(LEMSBase):
             else:
                 ct.parameters[parameter.name] = base_ct.parameters[parameter.name]
             
+        merge_maps(ct.properties, base_ct.properties)
+        
         merge_maps(ct.derived_parameters, base_ct.derived_parameters)
+        merge_maps(ct.index_parameters, base_ct.index_parameters)
         merge_maps(ct.constants, base_ct.constants)
         merge_maps(ct.exposures, base_ct.exposures)
         merge_maps(ct.requirements, base_ct.requirements)
+        merge_maps(ct.component_requirements, base_ct.component_requirements)
+        merge_maps(ct.instance_requirements, base_ct.instance_requirements)
         merge_maps(ct.children, base_ct.children)
         merge_maps(ct.texts, base_ct.texts)
         merge_maps(ct.links, base_ct.links)
@@ -373,8 +422,10 @@ class Model(LEMSBase):
 
         merge_maps(ct.simulation.runs, base_ct.simulation.runs)
         merge_maps(ct.simulation.records, base_ct.simulation.records)
+        merge_maps(ct.simulation.event_records, base_ct.simulation.event_records)
         merge_maps(ct.simulation.data_displays, base_ct.simulation.data_displays)
         merge_maps(ct.simulation.data_writers, base_ct.simulation.data_writers)
+        merge_maps(ct.simulation.event_writers, base_ct.simulation.event_writers)
 
     def fatten_component(self, c):
         """
@@ -398,6 +449,7 @@ class Model(LEMSBase):
 
         ### Resolve parameters
         for parameter in ct.parameters:
+            if self.debug: print("Checking: %s"%parameter)
             if parameter.name in c.parameters:
                 p = parameter.copy()
                 p.value = c.parameters[parameter.name]
@@ -411,12 +463,22 @@ class Model(LEMSBase):
                 raise ModelError("Parameter '{0}' not initialized for component '{1}'",
                                  parameter.name, c.id)
 
+        ### Resolve properties
+        for property in ct.properties:
+            property2 = property.copy()
+            fc.add(property2)
+            
         ### Resolve derived_parameters
         for derived_parameter in ct.derived_parameters:
             derived_parameter2 = derived_parameter.copy()
-            #derived_parameter2.numeric_value = self.get_numeric_value(constant2.value, constant2.dimension)
             fc.add(derived_parameter2)
             
+        ### Resolve derived_parameters
+        for index_parameter in ct.index_parameters:
+            raise ModelError("IndexParameter not yet implemented in PyLEMS!")
+            index_parameter2 = index_parameter.copy()
+            fc.add(index_parameter2)
+            
         ### Resolve constants
         for constant in ct.constants:
             constant2 = constant.copy()
@@ -449,8 +511,15 @@ class Model(LEMSBase):
                 raise ModelError("Path parameter '{0}' not initialized for component '{1}'",
                                  path.name, c.id)
 
+        if len(ct.component_requirements)>0:
+            raise ModelError("ComponentRequirement not yet implemented in PyLEMS!")
+        if len(ct.instance_requirements)>0:
+            raise ModelError("InstanceRequirement not yet implemented in PyLEMS!")
+
         ### Resolve component references.
         for cref in ct.component_references:
+            if cref.local:
+                raise ModelError("Attribute local on ComponentReference not yet implemented in PyLEMS!")
             if cref.name in c.parameters:
                 cref2 = cref.copy()
                 cid = c.parameters[cref.name]
@@ -466,6 +535,8 @@ class Model(LEMSBase):
             
         merge_maps(fc.exposures, ct.exposures)
         merge_maps(fc.requirements, ct.requirements)
+        merge_maps(fc.component_requirements, ct.component_requirements)
+        merge_maps(fc.instance_requirements, ct.instance_requirements)
         merge_maps(fc.children, ct.children)
         merge_maps(fc.texts, ct.texts)
         merge_maps(fc.links, ct.links)
@@ -533,6 +604,9 @@ class Model(LEMSBase):
                                  w.as_, fc.id)
             fc.structure.add(w2)
             
+        if len(ct.structure.tunnels) > 0:
+            raise ModelError("Tunnel is not yet supported in PyLEMS!");
+            
         for fe in ct.structure.for_eachs:
             fc.structure.add_for_each(fe)
             
@@ -639,8 +713,12 @@ class Model(LEMSBase):
 
         for mi in ct.structure.multi_instantiates:
             try:
-                mi2 = MultiInstantiate(fc.component_references[mi.component].referenced_component,
-                                       int(fc.parameters[mi.number].numeric_value))
+                if mi.component:
+                    mi2 = MultiInstantiate(component=fc.component_references[mi.component].referenced_component,
+                                           number=int(fc.parameters[mi.number].numeric_value))
+                else:
+                    mi2 = MultiInstantiate(component_type=fc.component_references[mi.component_type].referenced_component,
+                                           number=int(fc.parameters[mi.number].numeric_value))                    
             except:
                 raise ModelError("Unable to resolve multi-instantiate parameters for "
                                  "'{0}' in component '{1}'",
@@ -673,6 +751,15 @@ class Model(LEMSBase):
                                  fc.id)
             fc.simulation.add(record2)
 
+        for event_record in ct.simulation.event_records:
+            try:
+                event_record2 = EventRecord(fc.paths[event_record.quantity].value,
+                                 fc.texts[event_record.eventPort].value)
+            except:
+                raise ModelError("Unable to resolve simulation event_record parameters in component '{0}'",
+                                 fc.id)
+            fc.simulation.add(event_record2)
+
         for dd in ct.simulation.data_displays:
             try:
                 dd2 = DataDisplay(fc.texts[dd.title].value,
@@ -697,6 +784,20 @@ class Model(LEMSBase):
                                  fc.id)
             fc.simulation.add(dw2)
                 
+        for ew in ct.simulation.event_writers:
+            try:
+                path = '.'
+                if fc.texts[ew.path] and fc.texts[ew.path].value:
+                    path = fc.texts[ew.path].value
+                
+                ew2 = EventWriter(path,
+                                 fc.texts[ew.file_name].value,
+                                 fc.texts[ew.format].value)
+            except:
+                raise ModelError("Unable to resolve simulation writer parameters in component '{0}'",
+                                 fc.id)
+            fc.simulation.add(ew2)
+                
             
     def get_numeric_value(self, value_str, dimension = None):
         """
