@@ -116,7 +116,11 @@ class LEMSFileParser(LEMSBase):
                                                 'componentreference',
                                                 'exposure', 'eventport',
                                                 'fixed', 'link', 'parameter',
+                                                'property',
+                                                'indexparameter',
                                                 'path', 'requirement',
+                                                'componentrequirement',
+                                                'instancerequirement',
                                                 'simulation', 'structure',
                                                 'text', 'attachments',
                                                 'constant', 'derivedparameter']
@@ -141,12 +145,13 @@ class LEMSFileParser(LEMSBase):
                                             'eventconnection',
                                             'foreach',
                                             'multiinstantiate',
-                                            'with']
+                                            'with',
+                                            'tunnel']
                                        
         self.valid_children['foreach'] = ['foreach', 'eventconnection']     
                                             
-        self.valid_children['simulation'] = ['record', 'run',
-                                             'datadisplay', 'datawriter']
+        self.valid_children['simulation'] = ['record', 'eventrecord', 'run',
+                                             'datadisplay', 'datawriter', 'eventwriter']
 
         self.tag_parse_table = dict()
         #self.tag_parse_table['assertion'] = self.parse_assertion
@@ -155,11 +160,13 @@ class LEMSFileParser(LEMSBase):
         self.tag_parse_table['childinstance'] = self.parse_child_instance
         self.tag_parse_table['children'] = self.parse_children
         self.tag_parse_table['component'] = self.parse_component
-        self.tag_parse_table['componentreference'] = self.parse_component_reference
+        self.tag_parse_table['componentreference'] = self.parse_component_reference   
+        self.tag_parse_table['componentrequirement'] = self.parse_component_requirement
         self.tag_parse_table['componenttype'] = self.parse_component_type
         self.tag_parse_table['constant'] = self.parse_constant
         self.tag_parse_table['datadisplay'] = self.parse_data_display
         self.tag_parse_table['datawriter'] = self.parse_data_writer
+        self.tag_parse_table['eventwriter'] = self.parse_event_writer
         self.tag_parse_table['derivedparameter'] = self.parse_derived_parameter
         self.tag_parse_table['derivedvariable'] = self.parse_derived_variable
         self.tag_parse_table['conditionalderivedvariable'] = self.parse_conditional_derived_variable
@@ -173,6 +180,7 @@ class LEMSFileParser(LEMSBase):
         self.tag_parse_table['fixed'] = self.parse_fixed
         self.tag_parse_table['foreach'] = self.parse_for_each
         self.tag_parse_table['include'] = self.parse_include
+        self.tag_parse_table['indexparameter'] = self.parse_index_parameter
         self.tag_parse_table['kineticscheme'] = self.parse_kinetic_scheme
         self.tag_parse_table['link'] = self.parse_link
         self.tag_parse_table['multiinstantiate'] = self.parse_multi_instantiate
@@ -181,10 +189,13 @@ class LEMSFileParser(LEMSBase):
         self.tag_parse_table['onevent'] = self.parse_on_event
         self.tag_parse_table['onstart'] = self.parse_on_start
         self.tag_parse_table['parameter'] = self.parse_parameter
+        self.tag_parse_table['property'] = self.parse_property
         self.tag_parse_table['path'] = self.parse_path
         self.tag_parse_table['record'] = self.parse_record
+        self.tag_parse_table['eventrecord'] = self.parse_event_record
         self.tag_parse_table['regime'] = self.parse_regime
         self.tag_parse_table['requirement'] = self.parse_requirement
+        self.tag_parse_table['instancerequirement'] = self.parse_instance_requirement     
         self.tag_parse_table['run'] = self.parse_run
         #self.tag_parse_table['show'] = self.parse_show
         self.tag_parse_table['simulation'] = self.parse_simulation
@@ -195,6 +206,7 @@ class LEMSFileParser(LEMSBase):
         self.tag_parse_table['text'] = self.parse_text
         self.tag_parse_table['timederivative'] = self.parse_time_derivative
         self.tag_parse_table['transition'] = self.parse_transition
+        self.tag_parse_table['tunnel'] = self.parse_tunnel
         self.tag_parse_table['unit'] = self.parse_unit
         self.tag_parse_table['with'] = self.parse_with
 
@@ -529,8 +541,13 @@ class LEMSFileParser(LEMSBase):
         else:
             self.raise_error('<ComponentReference> must specify a type for the ' +
                              'reference.')
+                             
+        if 'local' in node.lattrib:
+            local = node.lattrib['local']
+        else:
+            local = None
 
-        self.current_component_type.add_component_reference(ComponentReference(name, type_))
+        self.current_component_type.add_component_reference(ComponentReference(name, type_, local))
 
     def parse_component_type(self, node):
         """
@@ -635,6 +652,33 @@ class LEMSFileParser(LEMSBase):
 
         self.current_simulation.add_data_writer(DataWriter(path, file_path))
 
+    def parse_event_writer(self, node):
+        """
+        Parses <EventWriter>
+
+        @param node: Node containing the <EventWriter> element
+        @type node: xml.etree.Element
+        """
+
+        if 'path' in node.lattrib:
+            path = node.lattrib['path']
+        else:
+            self.raise_error('<EventWriter> must specify a path.')
+
+        if 'filename' in node.lattrib:
+            file_path = node.lattrib['filename']
+        else:
+            self.raise_error("Event writer for '{0}' must specify a filename.",
+                             path)
+                             
+        if 'format' in node.lattrib:
+            format = node.lattrib['format']
+        else:
+            self.raise_error("Event writer for '{0}' must specify a format.",
+                             path)
+
+        self.current_simulation.add_event_writer(EventWriter(path, file_path, format))
+
     def parse_derived_parameter(self, node):
         """
         Parses <DerivedParameter>
@@ -955,7 +999,7 @@ class LEMSFileParser(LEMSBase):
         @raise ParseError: Raised when the file to be included is not specified. 
         """
         if not self.include_includes:
-            print("Ignoring included LEMS file: %s"%node.lattrib['file'])
+            if self.model.debug: print("Ignoring included LEMS file: %s"%node.lattrib['file'])
         else:
 
             #TODO: remove this hard coding for reading NeuroML includes...
@@ -1169,6 +1213,102 @@ class LEMSFileParser(LEMSBase):
 
         self.current_component_type.add_parameter(parameter)
 
+    def parse_property(self, node):
+        """
+        Parses <Property>
+
+        @param node: Node containing the <Property> element
+        @type node: xml.etree.Element
+
+        @raise ParseError: Raised when the property does not have a name.
+        @raise ParseError: Raised when the property does not have a
+        dimension.
+        """
+
+        if self.current_component_type == None:
+            self.raise_error('Property can only be defined in ' +
+                             'a component type')
+
+        try:
+            name = node.lattrib['name']
+        except:
+            self.raise_error('<Property> must specify a name')
+
+        try:
+            dimension = node.lattrib['dimension']
+        except:
+            self.raise_error("Property '{0}' has no dimension",
+                             name)
+                             
+        default_value = node.lattrib.get('defaultvalue', None)
+        
+        property = Property(name, dimension, default_value=default_value)
+
+        self.current_component_type.add_property(property)
+        
+        
+    def parse_index_parameter(self, node):
+        """
+        Parses <IndexParameter>
+
+        @param node: Node containing the <IndexParameter> element
+        @type node: xml.etree.Element
+
+        @raise ParseError: Raised when the IndexParameter does not have a name.
+        """
+
+        if self.current_component_type == None:
+            self.raise_error('IndexParameters can only be defined in ' +
+                             'a component type')
+
+        try:
+            name = node.lattrib['name']
+        except:
+            self.raise_error('<IndexParameter> must specify a name')
+
+
+        index_parameter = IndexParameter(name)
+
+        self.current_component_type.add_index_parameter(index_parameter)
+        
+        
+    def parse_tunnel(self, node):
+        """
+        Parses <Tunnel>
+
+        @param node: Node containing the <Tunnel> element
+        @type node: xml.etree.Element
+
+        @raise ParseError: Raised when the Tunnel does not have a name.
+        """
+
+        try:
+            name = node.lattrib['name']
+        except:
+            self.raise_error('<Tunnel> must specify a name')
+        try:
+            end_a = node.lattrib['enda']
+        except:
+            self.raise_error('<Tunnel> must specify: endA')
+        try:
+            end_b = node.lattrib['enda']
+        except:
+            self.raise_error('<Tunnel> must specify: endB')
+        try:
+            component_a = node.lattrib['componenta']
+        except:
+            self.raise_error('<Tunnel> must specify: componentA')
+        try:
+            component_b = node.lattrib['componentb']
+        except:
+            self.raise_error('<Tunnel> must specify: componentB')
+
+
+        tunnel = Tunnel(name, end_a, end_b, component_a, component_b)
+
+        self.current_structure.add_tunnel(tunnel)
+        
+
     def parse_path(self, node):
         """
         Parses <Path>
@@ -1209,6 +1349,31 @@ class LEMSFileParser(LEMSBase):
 
         self.current_simulation.add_record(Record(quantity, scale, color, id))
 
+    def parse_event_record(self, node):
+        """
+        Parses <EventRecord>
+
+        @param node: Node containing the <EventRecord> element
+        @type node: xml.etree.Element
+        """
+
+        if self.current_simulation == None:
+            self.raise_error('<EventRecord> must be only be used inside a ' +
+                             'simulation specification')
+
+        if 'quantity' in node.lattrib:
+            quantity = node.lattrib['quantity']
+        else:
+            self.raise_error('<EventRecord> must specify a quantity.')
+
+        if 'eventport' in node.lattrib:
+            eventPort = node.lattrib['eventport']
+        else:
+            self.raise_error('<EventRecord> must specify an eventPort.')
+
+
+        self.current_simulation.add_event_record(EventRecord(quantity, eventPort))
+
     def parse_regime(self, node):
         """
         Parses <Regime>
@@ -1255,7 +1420,42 @@ class LEMSFileParser(LEMSBase):
             self.raise_error("Requirement \{0}' must specify a dimension.", name)
 
         self.current_component_type.add_requirement(Requirement(name, dimension))
+    
+    def parse_component_requirement(self, node):
+        """
+        Parses <ComponentRequirement>
 
+        @param node: Node containing the <ComponentRequirement> element
+        @type node: xml.etree.Element
+        """
+
+        if 'name' in node.lattrib:
+            name = node.lattrib['name']
+        else:
+            self.raise_error('<ComponentRequirement> must specify a name')
+
+        self.current_component_type.add_component_requirement(ComponentRequirement(name))
+    
+    def parse_instance_requirement(self, node):
+        """
+        Parses <InstanceRequirement>
+
+        @param node: Node containing the <InstanceRequirement> element
+        @type node: xml.etree.Element
+        """
+
+        if 'name' in node.lattrib:
+            name = node.lattrib['name']
+        else:
+            self.raise_error('<InstanceRequirement> must specify a name')
+
+        if 'type' in node.lattrib:
+            type = node.lattrib['type']
+        else:
+            self.raise_error("InstanceRequirement \{0}' must specify a type.", name)
+
+        self.current_component_type.add_instance_requirement(InstanceRequirement(name, type))
+    
     def parse_run(self, node):
         """
         Parses <Run>
@@ -1495,9 +1695,14 @@ class LEMSFileParser(LEMSBase):
 
         if 'instance' in node.lattrib:
             instance = node.lattrib['instance']
+            list = None
+            index = None
+        elif 'list' in node.lattrib and 'index' in node.lattrib:
+            instance = None
+            list = node.lattrib['list']
+            index = node.lattrib['index']
         else:
-            self.raise_error('<With> must specify a reference to target'
-                             'instance')
+            self.raise_error('<With> must specify EITHER instance OR list & index')
 
         if 'as' in node.lattrib:
             as_ = node.lattrib['as']
@@ -1505,4 +1710,5 @@ class LEMSFileParser(LEMSBase):
             self.raise_error('<With> must specify a name for the '
                              'target instance')
 
-        self.current_structure.add_with(With(instance, as_))
+        self.current_structure.add_with(With(instance, as_, list, index))
+
