@@ -106,6 +106,14 @@ class SimulationBuilder(LEMSBase):
 
         for parameter in component.parameters:
             runnable.add_instance_variable(parameter.name, parameter.numeric_value)
+            
+        
+        for property in component.properties:
+            print("\n\n*****************************************************************\n\n"+
+                  "   Property element is not stable in PyLEMS yet, see https://github.com/LEMS/pylems/issues/16\n\n"+
+                  "   Used in: %s\n\n"%property.toxml()+
+                  "*****************************************************************\n\n\n")
+            runnable.add_instance_variable(property.name, property.default_value)
 
         derived_parameter_code = []
         
@@ -161,19 +169,19 @@ class SimulationBuilder(LEMSBase):
             r = runnable.regimes[rn]
             suffix = '_regime_' + rn
 
-            if runnable.__dict__.has_key('update_state_variables' + suffix): 
+            if 'update_state_variables' + suffix in runnable.__dict__: 
                   r.update_state_variables = runnable.__dict__['update_state_variables' + suffix]
                   
-            if runnable.__dict__.has_key('update_derived_variables' + suffix): 
+            if 'update_derived_variables' + suffix in runnable.__dict__: 
                 r.update_derived_variables = runnable.__dict__['update_derived_variables' + suffix]
             
-            if runnable.__dict__.has_key('run_startup_event_handlers' + suffix): 
+            if 'run_startup_event_handlers' + suffix in runnable.__dict__: 
                 r.run_startup_event_handlers = runnable.__dict__['run_startup_event_handlers' + suffix]
             
-            if runnable.__dict__.has_key('run_preprocessing_event_handlers' + suffix): 
+            if 'run_preprocessing_event_handlers' + suffix in runnable.__dict__: 
                 r.run_preprocessing_event_handlers = runnable.__dict__['run_preprocessing_event_handlers' + suffix]
             
-            if runnable.__dict__.has_key('run_postprocessing_event_handlers' + suffix): 
+            if 'run_postprocessing_event_handlers' + suffix in runnable.__dict__: 
                 r.run_postprocessing_event_handlers = runnable.__dict__['run_postprocessing_event_handlers' + suffix]
 
         self.process_simulation_specs(component, runnable, component.simulation)
@@ -209,7 +217,7 @@ class SimulationBuilder(LEMSBase):
             r = runnable.regimes[regime.name]
             suffix = '_regime_' + regime.name
 
-            if runnable.__dict__.has_key('update_kinetic_scheme' + suffix): 
+            if 'update_kinetic_scheme' + suffix in runnable.__dict__: 
                 r.update_kinetic_scheme = runnable.__dict__['update_kinetic_scheme' + suffix]
 
         self.add_recording_behavior(component, runnable)
@@ -681,8 +689,14 @@ class SimulationBuilder(LEMSBase):
 
         if func == 'ln':
             return 'log'
-        if func == 'random':
+        elif func == 'random':
             return 'random.uniform'
+        elif func == 'H':
+            def heaviside_step(x):
+                if x < 0: return 0
+                elif x > 0: return 1
+                else: return 0.5 
+            return 'heaviside_step'
         else:
             return func
 
